"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useCssAndCx = exports.cssAndCxFactory = void 0;
require("./tools/Object.fromEntries");
const classnames_1 = require("./tools/classnames");
const serialize_1 = require("@emotion/serialize");
const utils_1 = require("@emotion/utils");
const useSemanticGuaranteeMemo_1 = require("./tools/useSemanticGuaranteeMemo");
const cache_1 = require("./cache");
exports.cssAndCxFactory = (() => {
    function merge(registered, css, className) {
        const registeredStyles = [];
        const rawClassName = (0, utils_1.getRegisteredStyles)(registered, registeredStyles, className);
        if (registeredStyles.length < 2) {
            return className;
        }
        return rawClassName + css(registeredStyles);
    }
    function cssAndCxFactory(params) {
        const { cache } = params;
        const css = (...args) => {
            const serialized = (0, serialize_1.serializeStyles)(args, cache.registered);
            (0, utils_1.insertStyles)(cache, serialized, false);
            return `${cache.key}-${serialized.name}`;
        };
        const cx = (...args) => merge(cache.registered, css, (0, classnames_1.classnames)(args));
        return { css, cx };
    }
    return { cssAndCxFactory };
})().cssAndCxFactory;
/** Will pickup the contextual cache if any */
function useCssAndCx() {
    const cache = (0, cache_1.useTssEmotionCache)();
    const { css, cx } = (0, useSemanticGuaranteeMemo_1.useSemanticGuaranteeMemo)(() => (0, exports.cssAndCxFactory)({ cache }), [cache]);
    return { css, cx };
}
exports.useCssAndCx = useCssAndCx;
//# sourceMappingURL=cssAndCx.js.map